
# function without parameter (typically for side effect)

def myfunction():
  pass # pass does nothing


# function with a single mandatory parameter

def greet(name):
  print(f"Hello {name}")

# greet() # this would be an error
greet("Bob")  # this would print Hello Bob


# functions with multiple mandatory positional parameters
def multigreet(name, n):
  for i in range(n):
    print(f"Hello {name}")


# function with arbitrary number of paramters
def myprint(*words):
  for word in words:
    print(word) # prints each name on one line
  print(" ".join(words)) # just for fun, prints them all on one line

myprint("Joe", "Al", "Mo")

# positional (mandatory) and optional
def myprint(firstword, *morewords):
  pass

# keyword arguments with default values
def myprintkw(name="", n=0):
  for i in range(n):
    print(name) 

myprintkw(name="Joe", n=9)

# keyword arguments with default values and arbitrary extras
def myprintkwext(name="", n=0, **kwargs):
  for i in range(n):
    print(name) 
  print(kwargs)

# optional parameters, plus keyword arguments with default values and arbitrary extras
def myprintkwextopt(name="", n=0, *args, **kwargs):
  for i in range(n):
    print(name) 
  print(kwargs)



# Extension: type hints
#See: https://docs.python.org/3/library/typing.html
# TLDR: Just the functions section: https://mypy.readthedocs.io/en/stable/cheat_sheet_py3.html#functions
# canonical: https://typing.readthedocs.io/en/latest/spec/type-system.html


# Value errors
x = 0
if x < 100:
  raise ValueError("number is too low")

# Future reading: handling exceptions


# Future reading: asyncio functions


# Future reading: test driven development




