# 202409 COMP 150 Notes

## Resources

- [Git Primer](https://cisweb.ufv.ca/~i.janzenc/201809COMP152AB1/notes.html#_git_reference_supplement)
- [Markdown Primer](https://daringfireball.net/projects/markdown/)
- [Short Git Guide](https://rogerdudler.github.io/git-guide/)
- [ufv.syzygy.ca](https://ufv.syzygy.ca/)

### To generate a ssh keypair and display the pubkey
```
ssh-keygen
cat ~/.ssh/id_rsa.pub
```

OR

```
ssh-keygen -t ecdsa
cat ~/.ssh/id_ecdsa.pub
```
Next, copy and paste the output into the ssh keys section of your profile on https://sc-gitlab.ufv.ca

OR on a Windows machine in our computer lab (ie, CEP 0024)
```
R:
type .ssh\id_rsa.pub
```
*nb:* If you are on your own computer, you don't need to do the `R:` step.
*nb:* If you are using a lab computer on campus, you may need to first create the ``.ssh`` directory in the ``R:`` drive.


## How are Python programs/scripts invoked?

* Windows: File extensions are associated with a program
* Others: Typically looks at the first few bytes of the file (aka the "magic number")

Setting up the shebang line:

```
#!/usr/bin/env python

# your code goes here
print("Hello world!")

```

Setting up the executble bit (Mac/Linux)
```
chmod a+x yourprogram.py
```


Invoking your python program (Mac/Linux)
```
./yourprogram.py
```
OR, if you don't set up the shebang and execute bit as above:
```
python yourprogram.py
```

## Important

- Do not leave class before you are sure that you can write code and run it


## Variables

See: https://python.land/courses/python-introduction/lessons/variables

```
#!/usr/bin/env python3
#  Walking through the Python.land introduction material


# Variables have to be defined before they are used

# print(x)        # error if x is not declared/defined

result = 3 * 5
print(result)     # prints 15

var1 = result     # create a new variable called var1, and copy from result
result = 5

print(var1)       # What would this print? Answer: 15

# Duck Typing
x = "hello"
x = 5
x = 5 / 2
print(x)          # What would this print? Answer: 2.5

# operators
# multiplication
x = 5 * 3
# division
x = 5 / 2
# See also = python2 did integer division (with truncation)
# https://onecompiler.com/python2/42rvqgacy


# naming conventions
# Similar C - all lowercase for variables, include numbers if appropriate but not as first character, underscores are ok, dashes and other symbols are not ok

# 1username     # NOT ok because it starts with a number
# user-name     # NOT ok because it has a dash (hyphen)
# username      # fine, but some people might prefer word separation
# user_name     # fine, to use underscore to separate words
# userName      # fine/legal, but frowned upon in some circles (camel casing)
# _username     # fine/legal but has special visibility side effect/meaning

# for python and c, we expect to use up 80 columns of width, so use small vars
# for Java and C# and even sometimes C++, 120 columns may be acceptable

```

## Strings

see: https://python.land/introduction-to-python/strings

```
x = 'this is a string'
x = "this is also a string"
x = '-'*10        # this produces '----------'
x = '-' + '-'     # this produces '--'
# x = '--' - '-'   # this would cause an error

# escaping quotes

# x = "This line's "string" is outrageous!"  # this is an error
x = "This line's \"string\" is outrageous!"  # this works
x = 'This line\'s "string" is outrageous!'  # this works too
x = """This line's "string" is outrageous!"""  # this works, inlcuding multiline
x = """
this variable crosses multiple lines
this variable crosses multiple lines
this variable crosses multiple lines
this variable crosses multiple lines
"""
x = "this variable crosses multiple lines\nthis variable crosses multiple lines\n this variable crosses multiple lines\n this variable crosses multiple lines
"
x = "Hello\n"
x = """Hello
"""

# also have to know how to escape the escape (\)

x = "this is the backslash: \\\\" (produces: this is the backslash: \\)
# others: \t (tab) and carriage return \r
```

# strings are utf-8 encoded in python

```
x = "كيف حالك"
print(x)
```

# see also Chinook: https://dscorbett.github.io/duployan-test/
# font: https://github.com/dscorbett/duployan-test/tree/gh-pages/assets/fonts
# Kal-tash (Cultas, means useless) - may not appear correctly if you don't have the font
# see also: https://thewalimap.ca/html/panels/placenames.xhtml
x = "𛰅𛱁𛰆‌𛰃𛱁𛰛"


### f-strings

```
x = 2 
print("the value is " + 2)

# instead...
print(f"the value is {x}")
print(f"the value is {x*2}")


```
**NB:** See also [interpolation and string formatting](https://realpython.com/python-string-interpolation/)


## Booleans

```
x = True
if x:
  print('x is true')
  print('inside the block')

print('outside of block')

```

## Comparators

Evalute to a True or False (ie, >, >=, <, <=, !=, ==)


Loops: 
```
i = 0
while i < 10:
  i = i + 1
  print(i)

for x in range(1,11):   # overs a half-open interval [1,11)
  print(x)


for i in range(65,65+26):
  print(chr(i), chr(i+32))

# looping over a list
mylist = [0, 1, 5, 11, "hello"]

for item in mylist:
  print(item)

for item in mylist if item > 4:
  print(item)

print(mylist[4])  # prints: "hello" (recall zero-based indexing)

# writing the code to count to 10 and say we are done

for i in range(1,11):
  print(i)
print("done")


# make it re-usable:
NB: COMP150ON2 placeholder


i = 2

def count_and_done(from,to):
  for i in range(from, to):
    print(i)
  print(done)

count_and_done(1,11)
print(i) # still prints 2


```

## Dictionaries

```
things = {"Carl", "Ringo"}

grades = {
  "Carl": "C+",
  "Ringo": "A"
}
name = "Carl"
print(f"The grade for {name} is {grades[name]},")
print(f"...and the grade for Ringo is {grades["Ringo"]}")

grades["Mo"] = "A+"     # add an entry for Mo
del grades["Ringo"]     # remove the entry for Ringo

### Aside  - Exceptions for another day (try/catch)


```

## Integers
Be familiar with ALL of this page: https://python.land/python-data-types/python-integer


## Iterators and Iterables

From Python Land tutorial: https://python.land/deep-dives/python-iterator
```
>> my_iterable = range(1, 3)
>>> my_iterator = my_iterable.__iter__()
>>> my_iterator.__next__()
1
>>> my_iterator.__next__()
2
>>> my_iterator.__next__()
Traceback (most recent call last):
  File "<stdin>", line 1, in <module>
StopIteration
>>>
```


## Next:
  pre-read the following:
  - Classes and Object: https://python.land/objects-and-classes
  - Venvs and Package Mangement: https://python.land/virtual-environments
  next class:
  - Assignment 1 handed out


