# Setting Up Your Development Environment for COMP 150 

**General note**
When you see an example command, sometimes the usage instructions
have required and optional parameters. Required parameters look `<likethis>`
and optional paramters look `[likethis]`.

Example: the line below indicates that directory name is required
```bash
cd <directoryname>
```
This means you would do the following:
```bash
cd yourdirectory
```



This guide will walk you through setting up your development environment
for the first time. We'll cover how to generate SSH keys, add them to
your GitLab profile on `sc-gitlab.ufv.ca`, how to create and navigate
directories using the terminal on UFV Syzygy, and compare two common Git
workflows.

## 1. Working with Directories on UFV Syzygy 

You'll be using the terminal to create folders (directories) for
organizing your assignments. If you've never used a terminal before,
this section will help you get started.

### Accessing the Terminal on UFV Syzygy 

1.  **Log in** to [UFV Syzygy](https://ufv.syzygy.ca) with your UFV
    credentials.

2.  Once logged in, open the terminal by clicking **Terminal** from the
    launcher (under \"Other\") or from the top menu (File \> New \>
    Terminal).

### Basic Terminal Commands 

The terminal is a text-based way to interact with your computer. Here
are some basic commands to help you create and navigate directories:

#### 1.1 Check Your Current Directory

When you first open the terminal, you'll be in your home directory. To
see where you are, type:
```bash 
    pwd
```
This will show the current folder (directory) you are in.

#### 1.2 List Directory Contents

To see the files and folders in your current directory, type:

```bash 
    ls
```
#### 1.3 Create a New Directory

To create a new folder for your COMP 150 assignments:

```bash 
    mkdir comp150
```
#### 1.4 Navigate to a Directory

To move into the `comp150` directory you just created:

```bash 
    cd comp150
```
#### 1.5 Create Subdirectories for Assignments

**NB**: It is far more preferable to create the repository on sc-gitlab.ufv.ca first and clone, rather than creating the assignment folder manually as described below.

Inside the `comp150` folder, create a folder for Assignment 1:

```bash 
    mkdir assignment1
    cd assignment1
```
#### 1.6 Move Up a Directory

If you ever want to move up one level in the directory structure:

```bash 
    cd ..
```

### Example Folder Setup for Assignments 

1.  Create a main directory:

```bash 
            mkdir comp150
            cd comp150
```
2.  Create a subdirectory for your first assignment:

```bash 
            mkdir assignment1
            cd assignment1
```
Now you're in the right folder to start working on your assignment!

## 2. Generate SSH Keys (ECDSA) 

Using SSH keys is a secure way to interact with GitLab without needing
to enter your password each time. We'll use **ECDSA** (Elliptic Curve
Digital Signature Algorithm) for added security.

### 2.1 Generate an ECDSA SSH Key 

1.  Open the terminal on Syzygy or your local machine.

2.  Run the following command to generate an SSH key:
```bash
            ssh-keygen -t ecdsa
```
3.  When prompted for a file location, press `Enter` to accept the
    default location.

4.  You'll be asked for a passphrase (optional but recommended).

### 2.2 Copy the SSH Public Key 

To copy the public part of your key to your clipboard:

```bash 
    cat ~/.ssh/id_ecdsa.pub
```

### 2.3 Add SSH Key to GitLab 

1.  Log in to [sc-gitlab.ufv.ca](https://sc-gitlab.ufv.ca).

2.  Go to **Profile** \> **SSH Keys**.

3.  Paste your SSH key and click **Add key**.

## 3. Set Up Your Git Workflow 

Once your SSH key is added, you're ready to start using Git. Below are
two common workflows for creating and managing your repositories.

### Workflow 1: Create the Repository on GitLab First 

1.  **Create the repository on GitLab:**

    -   Log in to [sc-gitlab.ufv.ca](https://sc-gitlab.ufv.ca) and click
        **New Project**.

    -   Choose **Create blank project** and name it (e.g.,
        `202409comp150on1_assignment1`).

    -   Uncheck \"Initialize repository with README\" and click **Create
        project**.

2.  **Clone the repository locally:** In the terminal on Syzygy, run:
```bash
            git clone git@sc-gitlab.ufv.ca:<your-username>/<your-repo-name>.git
```
3.  **Start working:**

```bash
            cd <your-repo-name>
            echo "# My Project" > README.md
            git add README.md
            git commit -m "Initial commit"
            git push origin main
```

### Workflow 2: Create the Repository Locally First 

1.  **Create the repository locally:** In the terminal, run:

```bash
            mkdir <your-repo-name>
            cd <your-repo-name>
            git init
```

2.  **Add and commit files:**

```bash
            echo "# My Project" > README.md
            git add README.md
            git commit -m "Initial commit"
```

3.  **Create an empty repository on GitLab:**

    -   Create a new project on
        [sc-gitlab.ufv.ca](https://sc-gitlab.ufv.ca) but **leave it
        empty** (NO README FILE).

4.  **Link the local repository to GitLab:**

```bash
            git remote add origin git@sc-gitlab.ufv.ca:<your-username>/<your-repo-name>.git

```

5.  **Push your local work to GitLab:**

```bash
            git push -u origin main
```

**NB**: If you get a `incompatible histories` error, it is likely because you created a non-empty repository on `sc-gitlab`.

**NB**: The very first time you interact with the remote, you will get an error about the
authentictiy of the host:
```bash
The authenticity of host 'sc-gitlab.ufv.ca (198.162.116.21)' can't be established.
```
You **MUST** answer `yes`. If you just press enter, the command will fail.


## 4. Give Developer Access to `janzenc` 

Once your repository is created on GitLab, give access to your
instructor:

1.  Open your project on GitLab.

2.  Go to **Settings** \> **Members**, and select **Invite** from top of screen.

3.  Search for `janzenc`, set their role to **Developer**.

## Comparing the Two Git Workflows 

  **GitLab First Workflow**                                    **Local First Workflow**
  ------------------------------------------------------------ -------------------------------------------------------------
  **Best for beginners**: GitLab sets up everything for you.   **More control**: Start coding locally and push when ready.
  Clone from GitLab to start.                                  Manually link local repository to GitLab.
  Good for immediate collaboration.                            Great if you prefer working locally first.

## Basic Terminal Commands Cheat Sheet 

    **Command**    **Description**
  ---------------- -------------------------------------------------
       `pwd`       Print working directory (shows current folder)
        `ls`       List files and folders in the current directory
   `mkdir <name>`  Create a new directory with the given name
    `cd <name>`    Move into a directory
      `cd ..`      Move up one level in the directory tree
   `touch <file>`  Create a new, empty file



## To get out of vim: 
```
  [esc], :q! [enter]
```
 (or google "how to exit vim")

**NB**:  for future: do not use vim unless you have spent the time to go through vimtutor

## Nano basics

 The caret `^` symbol indicates `[ctrl]` as in `^X` means `[ctrl]+X`


## TODO: 
 - add an explanation of what to do when you have the `incompatible histories` error.
  TLDR explanation: start over with a new `sc-gitlab.ufv.ca` repository and then follow the instructions for pushing an existing repository.
