# Session 2

## Introduction to Python 

This Section is modeled after the 
tutorial on [Python Land](https://python.land/introduction-to-python).
For more detailed information, see the above link.

### 1. The Python REPL 

The Python REPL (Read-Eval-Print Loop) is an interactive shell where you
can write and run Python code line by line. It is a great tool for
testing small code snippets and learning Python interactively.

You can access the REPL by typing `python` (or `python3` depending on
your setup) in your terminal.

    >>> print("Hello, World!")
    Hello, World!

### 2. Variables 

Variables in Python are containers for storing data values. Python is
dynamically typed, so you do not need to declare a variable type
explicitly.

    x = 10
    name = "Alice"

### 3. Strings 

Strings are sequences of characters enclosed in single or double quotes.
You can manipulate strings using various methods like concatenation or
slicing.

    greeting = "Hello"
    name = "Bob"
    message = greeting + " " + name
    print(message)  # Outputs: Hello Bob

### 4. The `print()` Function 

The `print()` function outputs data to the screen. It can print strings,
variables, or any combination of them.

    print("This is a message")
    print("Sum:", 5 + 3)

### 5. Booleans 

Booleans represent `True` or `False` values. They are often used in
conditional statements or comparisons.

    is_active = True
    if is_active:
        print("The user is active")

### 6. Loops 

Loops allow you to repeat blocks of code. Python has `for` loops and
`while` loops.

**For Loop Example**:


```python
    for i in range(5):
        print(i)
```

**While Loop Example**:


```python
    count = 0
    while count < 3:
        print(count)
        count += 1
```

### 7. Functions 

Functions in Python allow you to reuse blocks of code. You define a
function with the `def` keyword and call it by its name.

```python
    def greet(name):
        return "Hello, " + name

    print(greet("Alice"))
```

Functions are called for their side effect (ie, printing something to the screen)
and/or their return value (ie, returning the result of a string manipulation or computation)


### 8. Dictionaries 

Dictionaries store data in key-value pairs. They are useful when you
want to associate a key with a value.

```python
    person = {"name": "John", "age": 30}
    print(person["name"])  # Outputs: John
```

Dictionaries are unordered, in that there is 
no first, next, etc concept in a dictionary.

```python
  { "name": "John",
    "age": 30 
  }
```
In the example, `name` and `age` belong to a set but their order is indeterminate

```python
["John", 30]
```
In this is example, the name is in position 0, and the age is in position 1.



A List, by contrast, has specific ordering.
Elements in a Dictionary are accessed by a key, 
while elements in a List are accessed by their position.


### 9. Numbers 

Python supports both integers and floating-point numbers (decimals).
Arithmetic operations are straightforward.

```python
    a = 10
    b = 2.5
    result = a + b
    print(result)  # Outputs: 12.5

    x = 5 / 2
    print(x)  # outputs 2.5 (would output 2 in C )

```




### 10. Iterators 

Iterators allow you to traverse through a sequence (like a list or a
string) one element at a time. Every `for` loop in Python uses an
iterator internally.

    my_list = [1, 2, 3]
    for item in my_list:
        print(item)

