# Example 1: \"Hello, World!\" in C (detail)
```c
    #include <stdio.h>  // This line includes the standard input/output library

    int main() {        // Entry point for the program
        printf("Hello, World!\n");  // Calls the printf function to output "Hello, World!" followed by a newline
        return 0;       // Indicates that the program finished successfully
    }
```
**Notes**:

-   `#include <stdio.h>`: This line imports the Standard Input/Output
    library, which contains the `printf` function used to print output
    to the terminal.

-   `int main() {}`: Every C program starts executing from the `main`
    function. The return type is `int`, meaning the program should
    return an integer.

-   `printf("Hello, World!`\
    `n");`: This function prints the string \"Hello, World!\" to the
    terminal. The\
    `n` is a newline character, meaning the cursor will move to the next
    line after printing.

-   `return 0;`: This indicates that the program finished without
    errors. Returning 0 from `main` is a common convention to indicate
    success.

# Example 2: Swap Function using Pointers on Scalars (C) 

```c
    #include <stdio.h>

    void swap(int *x, int *y) {  // Accepts two pointers to integers
        int temp = *x;           // Dereference x to access the value, and store it in temp
        *x = *y;                 // Assign the value pointed to by y into the location pointed to by x
        *y = temp;               // Assign the original value of x (stored in temp) to the location pointed to by y
    }

    int main() {
        int a = 10, b = 20;
        printf("Before swap: a = %d, b = %d\n", a, b);
        swap(&a, &b);            // Pass the addresses of a and b to the swap function
        printf("After swap: a = %d, b = %d\n", a, b);
        return 0;
    }
```

**Note**:

-   The `swap` function takes pointers to integers (`int *x` and
    `int *y`), allowing direct modification of the variables' values.

-   We dereference the pointers (`*x` and `*y`) to access and swap the
    actual values.

# Example 3: Find Maximum Value in an Array (C)

**This section is optional**

```c
    #include <stdio.h>

    int* find_max(int arr[], int size) {
        int *max = &arr[0];   // Pointer to the first element of the array
        for (int i = 1; i < size; i++) {
            if (arr[i] > *max) {
                max = &arr[i];  // Update max pointer to point to the new largest value
            }
        }
        return max;  // Return pointer to the maximum value
    }

    int main() {
        int arr[] = {10, 5, 7, 30, 20};
        int *max_ptr = find_max(arr, 5);
        printf("Max value: %d\n", *max_ptr);  // Dereference to print the value
        return 0;
    }
```

**Note**:

-   `find_max` returns a pointer to the largest element in the array.

-   Arrays in C are essentially pointers, so `arr[]` in function
    parameters is treated as a pointer.

# Example 4: Move Max to Beginning of Array with Pointer Swaps (C) 

**This section is optional**

```c
    #include <stdio.h>

    void swap(int *x, int *y) {
        int temp = *x;
        *x = *y;
        *y = temp;
    }

    int* find_max(int arr[], int size) {
        int *max = &arr[0];
        for (int i = 1; i < size; i++) {
            if (arr[i] > *max) {
                max = &arr[i];
            }
        }
        return max;
    }

    void move_max_to_beginning(int arr[], int size) {
        int *max_ptr = find_max(arr, size);
        if (max_ptr != &arr[0]) {
            swap(max_ptr, &arr[0]);  // Swap max value with the first element
        }
    }

    int main() {
        int arr[] = {10, 5, 7, 30, 20};
        printf("Before moving max: ");
        for (int i = 0; i < 5; i++) {
            printf("%d ", arr[i]);
        }
        printf("\n");

        move_max_to_beginning(arr, 5);

        printf("After moving max: ");
        for (int i = 0; i < 5; i++) {
            printf("%d ", arr[i]);
        }
        printf("\n");

        return 0;
    }
```

**Note**:

-   `move_max_to_beginning` finds the maximum value in the array and
    swaps it with the first element.

-   It calls the `swap` function, passing pointers to the elements that
    need to be swapped.
